@@ -5,6 +5,15 @@
 #include <string.h>
 #include "utils.h"
 
+/* Workaround for performance bug: https://github.com/libgit2/libgit2/issues/5725 */
+#if AT_LEAST_LIBGIT2(0, 26)
+#define USE_SUBMODULE_CACHE
+#endif
+
+#ifdef USE_SUBMODULE_CACHE
+#include <git2/sys/repository.h>
+#endif
+
 #ifndef GIT_ERROR_CALLBACK
 #define GIT_ERROR_CALLBACK GITERR_CALLBACK
 #endif
@@ -42,23 +51,23 @@ static const char *session_keyphrase(const char *set){
   return key;
 }
 
-static char* get_password(SEXP cb, const char *url, const char **username, int force_forget){
+static char* get_password(SEXP cb, const char *url, const char **username, int retries){
   if(!Rf_isFunction(cb))
     Rf_error("cb must be a function");
   int err;
-  SEXP call = PROTECT(Rf_lcons(cb, Rf_lcons(safe_string(url),
-                                   Rf_lcons(safe_string(*username),
-                                   Rf_lcons(Rf_ScalarLogical(force_forget),
-                                   R_NilValue)))));
+  SEXP call = PROTECT(Rf_lang4(cb,
+                               PROTECT(safe_string(url)),
+                               PROTECT(safe_string(*username)),
+                               PROTECT(Rf_ScalarInteger(retries))));
   SEXP res = PROTECT(R_tryEval(call, R_GlobalEnv, &err));
   if(err || !Rf_isString(res) || Rf_length(res) < 2){
-    UNPROTECT(2);
+    UNPROTECT(5);
     return NULL;
   }
   if(*username == NULL){
     *username = strdup(CHAR(STRING_ELT(res, 0)));
   }
-  UNPROTECT(2);
+  UNPROTECT(5);
   return strdup(CHAR(STRING_ELT(res, 1)));
 }
 
@@ -87,7 +96,7 @@ static void fin_git_repository(SEXP ptr){
   R_ClearExternalPtr(ptr);
 }
 
-static SEXP new_git_repository(git_repository *repo){
+SEXP new_git_repository(git_repository *repo){
   SEXP ptr = PROTECT(R_MakeExternalPtr(repo, R_NilValue, R_NilValue));
   R_RegisterCFinalizerEx(ptr, fin_git_repository, 1);
   Rf_setAttrib(ptr, R_ClassSymbol, Rf_mkString("git_repo_ptr"));
@@ -135,26 +144,6 @@ static void checkout_progress(const char *path, size_t cur, size_t tot, void *pa
   }
 }
 
-/* We only want to send the PAT to Github */
-static int url_is_github(const char *url, const char *user){
-  if(url == NULL)
-    return 0;
-  if(strstr(url, "http://github.com") == url)
-    return 1;
-  if(strstr(url, "https://github.com") == url)
-    return 1;
-  if(user){
-    char buf[4000];
-    snprintf(buf, 3999, "http://%s@github.com", user);
-    if(strstr(url, buf) == url)
-      return 1;
-    snprintf(buf, 3999, "https://%s@github.com", user);
-    if(strstr(url, buf) == url)
-      return 1;
-  }
-  return 0;
-}
-
 /* Examples: https://github.com/libgit2/libgit2/blob/master/tests/online/clone.c */
 static int auth_callback(git_cred **cred, const char *url, const char *username,
                                unsigned int allowed_types, void *payload){
@@ -213,16 +202,9 @@ static int auth_callback(git_cred **cred, const char *url, const char *username,
       print_if_verbose("Failed password authentiation %d times. Giving up\n", cb_data->retries - 1);
       cb_data->retries = 0;
     } else {
-      if(cb_data->retries == 0){
-        cb_data->retries++;
-        if(url_is_github(url, username) && getenv("GITHUB_PAT")){
-          print_if_verbose("Trying to authenticate with your GITHUB_PAT\n");
-          return git_cred_userpass_plaintext_new(cred, "git", getenv("GITHUB_PAT"));
-        }
-      }
       cb_data->retries++;
       print_if_verbose("Looking up https credentials for %s\n", url);
-      char *pass = get_password(cb_data->getcred, url, &username, cb_data->retries > 2);
+      char *pass = get_password(cb_data->getcred, url, &username, cb_data->retries);
       if(!username || !pass){
         print_if_verbose("Credential lookup failed\n");
         goto failure;
@@ -256,9 +238,9 @@ static git_strarray *files_to_array(SEXP files){
   return paths;
 }
 
-SEXP R_git_repository_init(SEXP path){
+SEXP R_git_repository_init(SEXP path, SEXP is_bare){
   git_repository *repo = NULL;
-  bail_if(git_repository_init(&repo, CHAR(STRING_ELT(path, 0)), 0), "git_repository_init");
+  bail_if(git_repository_init(&repo, CHAR(STRING_ELT(path, 0)), Rf_asLogical(is_bare)), "git_repository_init");
   return new_git_repository(repo);
 }
 
@@ -269,6 +251,9 @@ SEXP R_git_repository_open(SEXP path, SEXP search){
   } else {
     bail_if(git_repository_open(&repo, CHAR(STRING_ELT(path, 0))), "git_repository_open");
   }
+#ifdef USE_SUBMODULE_CACHE
+  git_repository_submodule_cache_all(repo);
+#endif
   return new_git_repository(repo);
 }
 
@@ -474,5 +459,7 @@ SEXP R_git_remote_ls(SEXP ptr, SEXP name, SEXP getkey, SEXP getcred, SEXP verbos
     SET_STRING_ELT(syms, i, safe_char(refs[i]->symref_target));
   }
   git_remote_free(remote);
-  return build_tibble(3, "ref", names, "symref", syms, "oid", oids);
+  SEXP out = build_tibble(3, "ref", names, "symref", syms, "oid", oids);
+  UNPROTECT(3);
+  return out;
 }
